/*
 * Copyright (c) 2018-2020 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice and this list of conditions.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice and  this list of  conditions in the  documentation and/or
 *      other materials provided with the distribution.
 *
 *                                      Anthony Mallet on Tue Mar 12 2018
 */
#ifndef H_MRSIM_PRIVATE
#define H_MRSIM_PRIVATE

#include <stdbool.h>
#include <stdint.h>
#include <time.h>

#include "libmrsim.h"

/* motor */
struct mrsim_motor_s {
  struct mrsimstate_motor_s state;

  bool servo;
  int32_t smc_bias, smc_gain, smc_err;
  struct timespec wdts;
};

/* IMU */
struct mrsim_imu_s {
  double w[3], a[3], B[3];
};

/* communication channel */
struct mrsim_pty_s {
  int fd;
  char device[512];
  uint8_t buf[128], r, w; /* read ring buffer */

  bool start;
  bool escape;
  uint8_t msg[64], len; /* last message */
};

/* timestamps */
struct mrsim_ts_s {
  double dt;
  struct timespec now, imu, mag, bat, mot;
};

/* simulation context */
struct mrsim_s {
  struct mrsim_pty_s * const pty;
  struct mrsim_param_s * const params;

  struct mrsim_motor_s * const motor;
  struct mrsimstate_wrench_s * const wrench;
  struct mrsimstate_body_s * const body;
  struct mrsim_imu_s * const imu;

  struct mrsim_ts_s * const ts;
};


#ifdef __cplusplus
extern "C" {
#endif

  void		mrsim_close_pty(struct mrsim_pty_s *pty);
  ssize_t	mrsim_recv_msg(struct mrsim_pty_s *pty, uint8_t **msg);
  int		mrsim_send_msg(const struct mrsim_pty_s *pty, const char *fmt,
                        ...);

  void		mrsim_motor_pwm(struct mrsim_motor_s *motor, const double pwm,
                        const struct timespec *ts);
  void		mrsim_motor_period(struct mrsim_motor_s *motor, const double wd,
                        const struct timespec *ts);
  void		mrsim_motor_control(const struct mrsim_param_s *p,
                        struct mrsim_motor_s *motor);
  void		mrsim_motor_model(const struct mrsim_param_s *p,
                        struct mrsim_motor_s *motor, const double dt);
  double	mrsim_motor_measure(const struct mrsimstate_motor_s *motor);
  uint8_t	mrsim_seqid(struct timespec tv, double period);

#ifdef __cplusplus
}
#endif

#endif /* H_MRSIM_PRIVATE */
